@@ -15,18 +15,18 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/calibr.c,v $
+ $Source: /usr/src/distr/ftape-1.13/RCS/calibr.c,v $
  $Author: bas $
  *
- $Revision: 1.20 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.22 $
+ $Date: 1994/06/23 04:51:23 $
  $State: Beta $
  *
  *      GP calibration routine for processor speed dependent
  *      functions.
  */
 
-static char RCSid[] = "$Id: calibr.c,v 1.20 1994/05/26 23:43:45 bas Beta bas $";
+static char RCSid[] = "$Id: calibr.c,v 1.22 1994/06/23 04:51:23 bas Beta $";
 
 #include <linux/errno.h>
 #include <linux/sched.h>
@@ -70,7 +70,7 @@ time_inb( void)
   cli();
   t0 = timestamp();
   for (i = 0; i < 1000; ++i) {
-    status = inb( FDC_STATUS);
+    status = inb( fdc.msr);
   }
   t1 = timestamp();
   restore_flags( flags);
@@ -78,17 +78,8 @@ time_inb( void)
   save_flags( flags);
   cli();
   t2 = timestamp();
-  for (i = 0; i < 100; ++i) {
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
-    status = inb( FDC_STATUS);
+  for (i = 0; i < 1000; ++i) {
+    status = inb( fdc.msr);
   }
   t3 = timestamp();
   restore_flags( flags);
