@@ -15,24 +15,25 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/fdc-io.c,v $
+ $Source: /usr/src/distr/ftape-1.13b/RCS/fdc-io.c,v $
  $Author: bas $
  *
- $Revision: 1.26 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.31 $
+ $Date: 1994/06/30 18:32:14 $
  $State: Beta $
  *
  *      This file contains the low-level floppy disk interface code
  *      for the QIC-40/80 tape streamer device driver.
  */
 
-static char RCSid[] = "$Id: fdc-io.c,v 1.26 1994/05/26 23:43:45 bas Beta bas $";
+static char RCSid[] = "$Id: fdc-io.c,v 1.31 1994/06/30 18:32:14 bas Beta $";
 
 #include <linux/errno.h>
 #include <linux/sched.h>
 #include <asm/system.h>
 #include <asm/io.h>
 #include <asm/dma.h>
+#include <asm/irq.h>
 
 #include "ftape.h"
 #include "fdc-io.h"
@@ -40,6 +41,8 @@ static char RCSid[] = "$Id: fdc-io.c,v 1.26 1994/05/26 23:43:45 bas Beta bas $";
 #include "ftape-io.h"
 #include "ftape-rw.h"
 #include "calibr.h"
+#include "fc-10.h"
+
 
 /*      Global vars.
  */
@@ -47,6 +50,7 @@ int ftape_unit = -1;
 int ftape_motor = 0;
 int current_cylinder = -1;
 fdc_mode_enum fdc_mode = fdc_idle;
+fdc_config_info fdc = { 0 };
 
 /*      Local vars.
  */
@@ -57,6 +61,15 @@ static int fdc_status;
 volatile byte fdc_head;         /* FDC head */
 volatile byte fdc_cyl;          /* FDC track */
 volatile byte fdc_sect;         /* FDC sector */
+static int fdc_data_rate = 0;   /* default rate = 500 Kbps */
+static int fdc_seek_rate = 14;  /* default rate = 2 msec @ 500 Kbps */
+#if defined FDC_BASE || !defined FLOPPY_HACK
+static void (*do_ftape)(void);
+#endif
+static int fdc_fifo_state = 0;  /* original fifo setting */
+static int fdc_lock_state = 0;  /* original lock setting */
+static int fdc_fifo_locked = 0; /* fifo & lock set ? */
+
 
 void
 fdc_catch_stray_interrupts( unsigned count)
@@ -83,7 +96,7 @@ fdc_wait( int usecs, byte mask, byte state)
   int count_1 = (fdc_calibr_count * usecs - 1) / fdc_calibr_time;
 
   do {
-    fdc_status = inb_p( FDC_STATUS);
+    fdc_status = inb_p( fdc.msr);
     if ((fdc_status & mask) == state) {
       return 0;
     }
@@ -135,7 +148,7 @@ fdc_write( byte data)
   if (fdc_wait( 150, FDC_DATA_READY_MASK, FDC_DATA_IN_READY) < 0) {
     return -ETIME;
   } else {
-    outb( data, FDC_DATA);
+    outb( data, fdc.fifo);
     return 0;
   }
 }
@@ -151,7 +164,7 @@ fdc_read( byte* data)
   if (fdc_wait( 150, FDC_DATA_READY_MASK, FDC_DATA_OUT_READY) < 0) {
     return -ETIME;
   } else {
-    *data = inb( FDC_DATA);
+    *data = inb( fdc.fifo);
     return 0;
   }
 }
@@ -171,7 +184,7 @@ fdc_command( byte* cmd_data, int cmd_len)
   fdc_usec_wait( RQM_DELAY);    /* wait for valid RQM status */
   save_flags( flags);
   cli();
-  fdc_status = inb( FDC_STATUS);
+  fdc_status = inb( fdc.msr);
   if ((fdc_status & FDC_DATA_READY_MASK) == FDC_DATA_IN_READY) {
     int retry = 0;
     fdc_mode = *cmd_data;       /* used by isr */
@@ -217,7 +230,7 @@ fdc_result( byte* res_data, int res_len)
 
   save_flags( flags);
   cli();
-  fdc_status = inb( FDC_STATUS);
+  fdc_status = inb( fdc.msr);
   if ((fdc_status & FDC_DATA_READY_MASK) == FDC_DATA_OUT_READY) {
     int retry = 0;
     while (count) {
@@ -379,15 +392,36 @@ fdc_motor( int motor)
   } else {
     TRACEx1( 4, "turning motor %d off", unit);
   }
-  outb_p( data, FDC_CONTROL);
+  outb_p( data, fdc.dor);
   ftape_sleep( 10 * MILLISECOND);
   TRACE_EXIT;
 }
 
+void
+fdc_set_data_rate( int rate)
+{
+  /* Select clock for fdc, must correspond with tape drive setting !
+   * This also influences the fdc timing so we must adjust some values.
+   */
+  switch (fdc.type) {
+  case fc10:
+  case i82077:
+    outb_p( rate & 0x03, fdc.dsr); /* set rate only, keep precomp, reset
+                                      and powerdown bits at 0 */
+    break;
+  case i8272:
+  default:
+    outb_p( rate, fdc.ccr);     /* set rate */
+    break;
+  }
+  fdc_data_rate = rate;
+  fdc_set_seek_rate( fdc_seek_rate); /* re-adjust for changed clock */
+}
+
 /*      Reset the floppy disk controller. Leave the ftape_unit selected.
  */
 void
-fdc_reset(void)
+fdc_reset( void)
 {
   TRACE_FUN( 8, "fdc_reset");
   int unit = FTAPE_UNIT;
@@ -395,24 +429,26 @@ fdc_reset(void)
   int st0;
   int i;
   int result;
+  int dummy;
 
   if (ftape_motor) {
     fdc_ctl |= FDC_MOTOR_0 << unit;
   }
-  outb_p( fdc_ctl, FDC_CONTROL); /* assert reset, keep unit selected */
+  outb_p( fdc_ctl, fdc.dor); /* assert reset, keep unit selected */
   fdc_usec_wait( 10 /* usec */); /* delay >= 14 fdc clocks */
   fdc_ctl |= FDC_RESET_NOT;
-  outb_p( fdc_ctl, FDC_CONTROL); /* release reset */
+  outb_p( fdc_ctl, fdc.dor); /* release reset */
   result = fdc_interrupt_wait( 1 * SECOND);
   if (result < 0) {
     TRACE( 1, "missing interrupt after reset");
   }
-  /* Select clock for fdc.  May need to be changed to select data rate.
-   */
-  outb_p( 0, FDC_VFO);          /* set high density (500 Kbps) mode */
+  fdc_set_data_rate( fdc_data_rate); /* keep original setting */
   fdc_usec_wait( 1000 /* usec */); /* don't know why, but needed */
   for (i = 0 ; i < 4 ; ++i) {   /* clear disk-change status */
-    fdc_sense_interrupt_status( &st0, &current_cylinder);
+    fdc_sense_interrupt_status( &st0, &dummy);
+    if (i == unit) {
+      current_cylinder = dummy;
+    }
   }
   fdc_set_seek_rate( 2);
   TRACE_EXIT;
@@ -424,7 +460,7 @@ fdc_reset(void)
 void
 fdc_disable( void)
 {
-  outb_p( FTAPE_UNIT, FDC_CONTROL);
+  outb_p( FTAPE_UNIT, fdc.dor);
 }
 
 /*      Specify FDC seek-rate
@@ -433,10 +469,27 @@ int
 fdc_set_seek_rate( int seek_rate)
 {
   byte in[ 3];
+  const int hut = 1;            /* minimal head unload time */
+  const int hlt = 1;            /* minimal head load time */
 
   in[ 0] = FDC_SPECIFY;
-  in[ 1] = ((16 - seek_rate) << 4) | 13 /* head_unload_time */;
-  in[ 2] = (1 /* head_load_time */ << 1) | 0 /* non_dma */;
+  switch (fdc_data_rate & 0x03) {
+  case 0:
+    in[ 1] = ((16 - (250 * seek_rate) / 500) << 4) | hut;
+    break;
+  case 1:
+    in[ 1] = ((16 - (300 * seek_rate) / 500) << 4) | hut;
+    break;
+  case 2:
+    in[ 1] = ((16 - (500 * seek_rate) / 500) << 4) | hut;
+    break;
+  case 3:
+    in[ 1] = ((16 - (1000 * seek_rate) / 500) << 4) | hut;
+    break;
+  default:
+  }
+  in[ 2] = (hlt << 1) | 0 /* non_dma */;
+  fdc_seek_rate = seek_rate;
 
   return fdc_command( in, 3);
 }
@@ -647,11 +700,11 @@ setup_fdc_and_dma( buffer_struct* buff, unsigned char operation)
    */
   save_flags( flags);
   cli();                      /* could be called from ISR ! */
-  disable_dma( FLOPPY_DMA);
-  clear_dma_ff( FLOPPY_DMA);
-  set_dma_mode( FLOPPY_DMA, dma_mode);
-  set_dma_addr( FLOPPY_DMA, (unsigned) buff->ptr);
-  set_dma_count( FLOPPY_DMA, SECTOR_SIZE * buff->sector_count);
+  disable_dma( fdc.dma);
+  clear_dma_ff( fdc.dma);
+  set_dma_mode( fdc.dma, dma_mode);
+  set_dma_addr( fdc.dma, (unsigned) buff->ptr);
+  set_dma_count( fdc.dma, SECTOR_SIZE * buff->sector_count);
 #ifdef GCC_2_4_5_BUG
   /*  This seemingly stupid construction confuses the gcc-2.4.5
    *  code generater enough to create correct code.
@@ -664,7 +717,7 @@ setup_fdc_and_dma( buffer_struct* buff, unsigned char operation)
     }
   }
 #endif
-  enable_dma( FLOPPY_DMA);
+  enable_dma( fdc.dma);
   /* Issue FDC command to start reading/writing.
    */
   out[ 0] = operation;
@@ -677,8 +730,8 @@ setup_fdc_and_dma( buffer_struct* buff, unsigned char operation)
   out[ 7] = 109;                /* Gap length. */
   out[ 8] = 0xff;		/* No limit to transfer size. */
   restore_flags( flags);
-  TRACEx4( 6, "c: 0x%02x, h: 0x%02x, s: 0x%02x, "
-          "cnt: 0x%02x", out[ 2], out[ 3], out[ 4], out[ 6]- out[ 4]+ 1);
+  TRACEx4( 6, "C: 0x%02x, H: 0x%02x, R: 0x%02x, cnt: 0x%02x",
+          out[ 2], out[ 3], out[ 4], out[ 6]- out[ 4]+ 1);
   result = fdc_command( out, 9);
   if (result != 0) {
     fdc_mode = fdc_idle;
@@ -689,3 +742,283 @@ setup_fdc_and_dma( buffer_struct* buff, unsigned char operation)
   return result;
 }
 
+int
+fdc_fifo_enable( void)
+{
+  TRACE_FUN( 8, "fdc_fifo_enable");
+  int result = 0;
+  byte cmd0[] = { 0x0e };       /* dumpreg */
+  byte cmd1[] = { 0x13, 0x00, 0x2b, 0x00 }; /* enable fifo, thr = 12 */
+  byte cmd2[] = { 0x94 };       /* lock */
+  byte stat;
+  byte reg[ 10];
+  int i;
+
+  if (!fdc_fifo_locked) {
+    /*  Dump fdc internal registers for examination
+     */
+    result = fdc_command( cmd0, NR_ITEMS( cmd0));
+    if (result < 0) {
+      TRACE( 2, "FDC dumpreg command failed, fifo unchanged");
+      result = -EIO;
+    } else {
+      /*  Now read fdc internal registers from fifo
+       */
+      for (i = 0 ; i < NR_ITEMS( reg) ; ++i) {
+        fdc_read( &reg[ i]);
+        TRACEx2( 5, "Register %d = 0x%02x", i, reg[ i]);
+      }
+      fdc_fifo_state = reg[ 8] & 0x20;
+      fdc_lock_state = reg[ 7] & 0x80;
+      if (fdc_fifo_state) {
+        TRACEx2( 1, "fifo already enabled, thresshold %d%s", reg[ 8] & 0x0f,
+                (fdc_lock_state) ? ", locked" : "");
+      }
+      /*  Enable fifo and set thresshold at 12 bytes for large latency
+       */
+      fdc_ready_wait( 100);
+      result = fdc_command( cmd1, NR_ITEMS( cmd1));
+      if (result < 0) {
+        TRACE( -1, "FDC configure command failed, fifo unchanged");
+        result = -EIO;
+      } else {
+        /*  Now lock configuration so reset will not change it
+         */
+        result = fdc_issue_command( cmd2, NR_ITEMS( cmd2), &stat, 1);
+        if (result < 0 || stat != 0x10) {
+          TRACEx1( -1, "FDC lock command failed, stat = 0x%02x", stat);
+          result = -EIO;
+        } else {
+          fdc_fifo_locked = 1;
+        }
+      }
+    }
+  }
+  TRACE_EXIT;
+  return result;
+}
+
+int
+fdc_probe( void)
+{
+  TRACE_FUN( 8, "fdc_probe");
+  byte cmd = FDC_VERSION;
+  byte stat;
+  int result;
+
+  /*  Try to find out what kind of fd controller we have to deal with
+   */
+  result = fdc_issue_command( &cmd, 1, &stat, 1);
+  if (result < 0) {
+    TRACE( -1, "Couldn't determine fdc version, assume 8272");
+    result = i8272;             /* safe choice */
+  } else {
+    if (stat == 0x90) {
+      TRACE( 2, "Type 82077 or compatible FDC found");
+      result = i82077;
+    } else if (stat == 0x80) {
+      TRACE( 2, "Type 8272A or compatible FDC found");
+      result = i8272;
+    } else {
+      TRACEx1( -1, "Unknown FDC type found: 0x%02x", stat);
+      result = i8272;
+    }
+  }
+  TRACE_EXIT;
+  return result;
+}
+
+void
+fdc_config_regs( unsigned fdc_base, unsigned fdc_irq, unsigned fdc_dma)
+{
+  fdc.irq = fdc_irq;
+  fdc.dma = fdc_dma;
+  fdc.sra = fdc_base;
+  fdc.srb = fdc_base + 1;
+  fdc.dor = fdc_base + 2;
+  fdc.tdr = fdc_base + 3;
+  fdc.msr = fdc.dsr = fdc_base + 4;
+  fdc.fifo = fdc_base + 5;
+  fdc.dir = fdc.ccr = fdc_base + 7;
+}
+
+/*  The code that follows will be a lot cleaner when the
+ *  irq and dma can be shared with the floppy driver.
+ */
+/*  If probing for a FC-10 controller the fdc base address, interrupt
+ *  and dma channel must be specified.
+ *  If using an alternate fdc controller, base address, interrupt and
+ *  dma channel must be specified.
+ */
+#if defined PROBE_FC10 && !defined FDC_BASE
+# error No FDC base address (FDC_BASE) specified in Makefile!
+#endif
+#if defined FDC_BASE && !defined FDC_IRQ
+# error No interrupt (FDC_IRQ) specified in Makefile!
+#endif
+#if defined FDC_BASE && !defined FDC_DMA
+# error No dma channel (FDC_DMA) specified in Makefile!
+#endif
+
+void
+fdc_config( void)
+{
+  TRACE_FUN( 8, "fdc_config");
+  static int already_done = 0;
+
+  if (!already_done) {
+#ifdef PROBE_FC10
+    fdc_config_regs( FDC_BASE, FDC_IRQ, FDC_DMA);
+    if (fc10_enable()) {
+      TRACE( 2, "FC-10 controller found");
+      fdc.type = fc10;
+      fdc.hook = &do_ftape;
+    } else {
+      TRACE( 2, "FC-10 controller not found");
+      fdc_config_regs( 0x3f0, 6, 2); /* back to std fdc again */
+#ifdef FLOPPY_HACK
+      fdc.hook = &do_floppy;
+#else
+      fdc.hook = &do_ftape;
+#endif
+    }
+#else
+# ifdef FDC_BASE
+    TRACE( 2, "Using fdc controller at alternate address");
+    fdc_config_regs( FDC_BASE, FDC_IRQ, FDC_DMA);
+    fdc.hook = &do_ftape;
+# else
+    TRACE( 2, "Using the standard fdc controller");
+    fdc_config_regs( 0x3f0, 6, 2); /* std fdc */
+#ifdef FLOPPY_HACK
+    fdc.hook = &do_floppy;
+#else
+    fdc.hook = &do_ftape;
+#endif
+# endif
+#endif
+  }
+  *(fdc.hook) = fdc_isr;          /* hook our handler in */
+  already_done = 1;
+  TRACE_EXIT;
+}
+
+#ifdef FLOPPY_HACK
+
+inline int
+fdc_grab_irq_and_dma( void)
+{
+  return 0;
+}
+
+inline int
+fdc_release_irq_and_dma( void)
+{
+  return 0;
+}
+
+#else
+
+static void ftape_interrupt( int unused)
+{
+  TRACE_FUN( 8, "ftape_interrupt");
+  void (*handler)(void) = *fdc.hook;
+
+  *fdc.hook = NULL;
+  if (handler) {
+    handler();
+  } else {
+    TRACE( -1, "Unexpected ftape interrupt");
+  }
+  TRACE_EXIT;
+}
+
+int
+fdc_grab_irq_and_dma( void)
+{
+  TRACE_FUN( 8, "fdc_grab_irq_and_dma");
+  int result = 0;
+  struct sigaction ftape_sig_action = {
+    ftape_interrupt, 0, SA_INTERRUPT, NULL
+  };
+
+  if (fdc.hook != &do_floppy) {
+    /*  Cannot use request_irq because we want a fast interrupt
+     *  handler instead of a normal one (see kernel/irq.c).
+     */
+    result = irqaction( fdc.irq, &ftape_sig_action);
+    if (result) {
+      TRACEx1( -1, "Unable to grab IRQ%d for ftape driver", fdc.irq);
+      result = -EIO;
+    } else {
+      result = request_dma( fdc.dma);
+      if (result) {
+        TRACEx1( -1, "Unable to grab DMA%d for ftape driver", fdc.dma);
+        free_irq( fdc.irq);
+        result = -EIO;
+      } else {
+        enable_irq( fdc.irq);
+      }
+    }
+  }
+  TRACE_EXIT;
+  return result;
+}
+
+int
+fdc_release_irq_and_dma( void)
+{
+  int result = 0;
+
+  if (fdc.hook != &do_floppy) {
+    disable_dma( fdc.dma);      /* just in case... */
+    free_dma( fdc.dma);
+    disable_irq( fdc.irq);
+    free_irq( fdc.irq);
+  }
+  return result;
+}
+
+#endif
+
+int
+fdc_init( void)
+{
+  TRACE_FUN( 8, "fdc_init");
+  int result = 0;
+
+  fdc_config();
+  if (fdc_grab_irq_and_dma() < 0) {
+    result = -EBUSY;
+  } else {
+    ftape_motor = 0;
+    fdc_catch_stray_interrupts( 1); /* one always comes */
+    TRACE( 5, "resetting fdc");
+    fdc_reset();                  /* init fdc & clear track counters */
+    if (fdc.type == i8272) {      /* default, means no fc-10 found */
+      fdc.type = fdc_probe();
+    }
+    if (fdc.type == i82077 || fdc.type == fc10) {
+      if (fdc_fifo_enable() >= 0) {
+        TRACE( 2, "FDC fifo successfully enabled");
+      }
+    }
+#ifdef SOFT_RESET
+    /*  The soft reset caused by the fdc_recalibrate command will always
+     *  set the new-cartridge status on Iomega make drives. This is probably
+     *  due to an QIC-117 spec interpretation error. Since we have to live
+     *  with it we'll try to prevent soft resets as much as possible.
+     *  The function of this call was to clear the track counter in the fdc
+     *  to zero. That might be done by the fdc_reset too (I'm not sure).
+     */
+    TRACE( 5, "recalibrating");
+    if (fdc_recalibrate() != 0) {
+      TRACE( 1, "recalibrate failed");
+      fdc_release_irq_and_dma();
+      result = -EIO;
+    }
+#endif
+  }
+  TRACE_EXIT;
+  return result;
+}
