@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /usr/src/distr/ftape-1.12/RCS/fdc-io.h,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/fdc-io.h,v $
  $Author: bas $
  *
- $Revision: 1.25 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.28 $
+ $Date: 1994/06/26 14:10:38 $
  $State: Beta $
  *
  *      This file contains the low level functions
@@ -47,11 +47,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #define FDC_SEEK        (FD_SEEK)
 #define FDC_SPECIFY     (FD_SPECIFY)
 #define FDC_RECALIBR    (FD_RECALIBRATE)
-
-#define FDC_STATUS      (FD_STATUS)
-#define FDC_DATA        (FD_DATA)
-#define FDC_CONTROL     (FD_DOR)
-#define FDC_VFO         (FD_DCR)
+#define FDC_VERSION     (FD_VERSION)
 
 #define FDC_STATUS_MASK (STATUS_BUSY | STATUS_DMA | STATUS_DIR | STATUS_READY)
 #define FDC_DATA_READY  (STATUS_READY)
@@ -74,6 +70,30 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #define FDC_MOTOR_0     (0x10)
 #define FDC_MOTOR_1     (0x20)
 
+typedef struct {
+  void (**hook)(void);          /* our wedge into the isr */
+  enum { i8272, i82077, fc10
+       } type;                  /* FDC type */
+  unsigned char irq;            /* FDC irq nr */
+  unsigned char dma;            /* FDC dma channel nr */
+  unsigned short sra;           /* Status register A (PS/2 only) */
+  unsigned short srb;           /* Status register B (PS/2 only) */
+  unsigned short dor;           /* Digital output register */
+  unsigned short tdr;           /* Tape Drive Register (82077SL-1 only) */
+  unsigned short msr;           /* Main Status Register */
+  unsigned short dsr;           /* Datarate Select Register (82077 only) */
+  unsigned short fifo;          /* Data register / Fifo on 82077 */
+  unsigned short dir;           /* Digital Input Register */
+  unsigned short ccr;           /* Configuration Control Register */
+} fdc_config_info;
+
+typedef enum {
+  fdc_data_rate_250 = 2,
+  fdc_data_rate_300 = 1,
+  fdc_data_rate_500 = 0,
+  fdc_data_rate_1000 = 3,
+} fdc_data_rate_type;
+
 typedef enum {
   empty = 0,
   reading,
@@ -108,6 +128,7 @@ extern int current_cylinder;    /* track nr the FDC thinks we're on */
 extern volatile byte fdc_head;  /* FDC head */
 extern volatile byte fdc_cyl;   /* FDC track */
 extern volatile byte fdc_sect;  /* FDC sector */
+extern fdc_config_info fdc;     /* FDC hardware configuration */
 
 /*
  *      fdc-io.c defined public functions
@@ -134,5 +155,8 @@ extern int fdc_recalibrate( void);
 extern void fdc_disable( void);
 extern int fdc_wait_calibrate( void);
 extern int fdc_sense_interrupt_status( int *st0, int *current_cylinder);
+extern void fdc_set_data_rate( int rate);
+extern int fdc_release_irq_and_dma( void);
+extern int fdc_init( void);
 
 #endif
