@@ -15,18 +15,18 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/fdc-isr.c,v $
+ $Source: /usr/src/distr/ftape-1.13b/RCS/fdc-isr.c,v $
  $Author: bas $
  *
- $Revision: 1.9 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.15 $
+ $Date: 1994/07/05 20:18:02 $
  $State: Beta $
  *
  *      This file contains the interrupt service routine and associated
  *      code for the QIC-40/80 tape streamer device driver.
  */
 
-static char RCSid[] = "$Id: fdc-isr.c,v 1.9 1994/05/26 23:43:45 bas Beta bas $";
+static char RCSid[] = "$Id: fdc-isr.c,v 1.15 1994/07/05 20:18:02 bas Beta bas $";
 
 #include <asm/io.h>
 #include <asm/dma.h>
@@ -99,9 +99,9 @@ decode_irq_cause( fdc_mode_enum fdc_mode, byte st[],
     break;
   case FDC_INT_ABNORMAL:
     TRACEx1( 5, "abnormal completion %s", *fdc_mode_txt);
-    TRACEx3( 6, "sr0: 0x%02x, sr1: 0x%02x, sr2: 0x%02x",
+    TRACEx3( 6, "ST0: 0x%02x, ST1: 0x%02x, ST2: 0x%02x",
             st[ 0], st[ 1], st[ 2]);
-    TRACEx4( 6, "c: 0x%02x, h: 0x%02x, s: 0x%02x, z: 0x%02x",
+    TRACEx4( 6, "C: 0x%02x, H: 0x%02x, R: 0x%02x, N: 0x%02x",
             st[ 3], st[ 4], st[ 5], st[ 6]);
     if (st[ 1] & 0x01) {
       if (st[ 2] & 0x01) {
@@ -167,8 +167,8 @@ determine_progress( buffer_struct* buff, error_cause cause)
   unsigned nr_not_xferred;
   unsigned dma_residue;
 
-  disable_dma( FLOPPY_DMA);
-  dma_residue = get_dma_residue( FLOPPY_DMA);
+  disable_dma( fdc.dma);
+  dma_residue = get_dma_residue( fdc.dma);
   if (cause != no_error || dma_residue > 0) {
     if (cause == no_error) {
       TRACEx1( 6, "unexpected DMA residue: 0x%04x", dma_residue);
@@ -179,7 +179,8 @@ determine_progress( buffer_struct* buff, error_cause cause)
   } else {
     nr_not_xferred = 0;
   }
-  if (cause != no_error && cause != data_crc_error && cause != overrun_error) {
+  if (cause != no_error && cause != data_crc_error &&
+      cause != overrun_error && cause != no_data_error) {
     if (nr_not_xferred > 0) {
       --nr_not_xferred;         /* skip error in sector not yet done */
     } else {
@@ -187,7 +188,7 @@ determine_progress( buffer_struct* buff, error_cause cause)
     }
   }
   if (nr_not_xferred > 0) {     /* compensate for sectors not transferred */
-    buff->sector_count-= nr_not_xferred;
+    buff->sector_count -= nr_not_xferred;
     /* bad_sector_map is already adjusted for the situation if there
      * were sector_count sectors done without error (calc_next_cluster()).
      */
@@ -195,15 +196,25 @@ determine_progress( buffer_struct* buff, error_cause cause)
   }
   /*  Update var's influenced by the DMA operation.
    */
-  buff->sector_offset += buff->sector_count;
-  buff->data_offset += buff->sector_count;
-  buff->ptr += buff->sector_count * SECTOR_SIZE;
-  buff->remaining -= buff->sector_count;
+  if (buff->sector_count != 0) {
+    buff->sector_offset += buff->sector_count;
+    buff->data_offset += buff->sector_count;
+    buff->ptr += buff->sector_count * SECTOR_SIZE;
+    buff->remaining -= buff->sector_count;
+  }
   if (cause != no_error) {
-    /* merge position of bad sector in bitmap
-     */
-    buff->crc_error_map |=  (1 << (buff->data_offset - 1));
-    TRACEx1( 4, "Error at sector offset %d", buff->data_offset - 1);
+    if (cause != no_data_error) {
+      /* merge position of bad sector in bitmap
+       */
+      if (buff->data_offset > 0) {
+        buff->crc_error_map |=  (1 << (buff->data_offset - 1));
+        TRACEx1( 4, "Error at sector offset %d", buff->data_offset - 1);
+      } else {
+        TRACEx1( 4, "Unexpected error at sector offset %d", buff->data_offset);
+      }
+    } else {
+      TRACEx1( 4, "Sector at offset %d not found", buff->data_offset);
+    }
   } else {
     TRACEx1( 5, "%d Sectors successfully transferred", buff->sector_count);
   }
@@ -277,7 +288,6 @@ continue_xfer( buffer_struct** p_buff, error_cause cause,
   byte fdc_op = (write) ? FDC_WRITE : FDC_READ;
 
   if (!skip) {
-    determine_progress( buff, cause);
     if (runner_status != running || (write && buff->status != writing) ||
         (!write && buff->status != reading)) {
       TRACEx2( 1, "unexpected runner/buffer state %d/%d",
@@ -357,7 +367,6 @@ retry_sector( buffer_struct** p_buff, error_cause cause,
     continue_xfer( p_buff, cause, write, 0);
   } else {
 #endif
-    determine_progress( buff, cause);
     TRACEx1( 5, "%s error, preparing for retry", (write) ? "write" : "read");
 #if 1
     runner_status = aborting;
@@ -399,13 +408,13 @@ fdc_isr( void)
   t0 = timestamp();
   if (isr_active) {
     TRACE( -1, "nested interrupt, not good !");
-    do_floppy = fdc_isr;        /* hook our handler into the fdc code again */
+    *fdc.hook = fdc_isr;        /* hook our handler into the fdc code again */
     TRACE_EXIT;
     return;
   }
   ++isr_active;
   sti();                        /* enables interrupts again */
-  status = inb_p( FDC_STATUS);
+  status = inb_p( fdc.msr);
   if (status & FDC_BUSY) {      /*  Entering Result Phase */
     hide_interrupt = 0;
     result = fdc_result( in, 7); /* better get it fast ! */
@@ -449,6 +458,7 @@ fdc_isr( void)
         } else {
           switch (cause) {
           case no_error: {
+            determine_progress( buff, cause);
             continue_xfer( &buff, cause, 0, 0);
             break;
           }
@@ -456,17 +466,30 @@ fdc_isr( void)
             int skip;
 
             /*  Tape started too far ahead of or behind the right sector.
+             *  This may also happen in the middle of a segment !
              */
             update_history( cause);
-            if (buff->data_offset > 0 &&
+            determine_progress( buff, cause);
+            if (buff->sector_count > 0) {
+              /*  Get here if at least one sector was read successfully
+               *  before some sector id couldn't be found.
+               */
+              TRACEx1( 5, "partial success: %d sectors done up to now",
+                      buff->sector_count);
+              skip = buff->sector_offset; /* retry error sector */
+            } else if (buff->data_offset > 0 &&
                 buff->crc_error_map & (1 << (buff->data_offset - 1))) {
+              /*  Get here if the sector id not found comes right
+               *  after a failing sector.
+               */
               TRACE( 5, "error in previous sector: resume problem");
-              if (buff->crc_error_map & ((1 << (buff->data_offset - 1)) - 1)) {
+              if (buff->data_offset > 1 &&
+                  buff->crc_error_map & ((1 << (buff->data_offset - 1)) - 1)) {
                 TRACE( 5, "not first error in this segment");
-                skip = buff->sector_offset - 1;
+                skip = buff->sector_offset - 1; /* retry error sector */
               } else {
                 TRACE( 5, "first error in this segment");
-                skip = buff->sector_offset; /* keep error */
+                skip = buff->sector_offset; /* skip error sector */
               }
             } else {
               TRACE( 5, "no previous error: resume problem");
@@ -482,8 +505,11 @@ fdc_isr( void)
           case id_am_error:
           case id_crc_error:
           case data_am_error: {
+            int first_error = (buff->crc_error_map == 0);
+
             update_history( cause);
-            if (buff->crc_error_map == 0) {
+            determine_progress( buff, cause);
+            if (first_error) {
               /*  Let ecc code correct this single error
                */
               TRACE( 4, "single am/crc error, let ecc handle it");
@@ -498,8 +524,11 @@ fdc_isr( void)
           }
           case data_crc_error:
           case overrun_error: {
+            int first_error = (buff->crc_error_map == 0);
+
+            determine_progress( buff, cause);
             update_history( cause);
-            if (buff->crc_error_map == 0) {
+            if (first_error) {
               if (cause == data_crc_error) {
                 TRACE( 4, "single crc error in data");
               } else {
@@ -516,6 +545,7 @@ fdc_isr( void)
             /*  Don't know why this could happen but find out.
              */
             TRACE( 1, "unexpected error");
+            determine_progress( buff, cause);
             retry_sector( &buff, cause, 0, 0);
             break;
           }
@@ -535,7 +565,7 @@ fdc_isr( void)
           fdc_head =
           fdc_sect = 0;
         }
-        TRACEx3( 6, "id read: c: 0x%02x, h: 0x%02x, s: 0x%02x",
+        TRACEx3( 6, "id read: C: 0x%02x, H: 0x%02x, R: 0x%02x",
                 fdc_cyl, fdc_head, fdc_sect);
         fdc_mode = fdc_idle;
         break;
@@ -563,14 +593,20 @@ fdc_isr( void)
         } else {
           switch (cause) {
           case no_error: {
+            determine_progress( buff, cause);
             continue_xfer( &buff, cause, 1, 0);
             break;
           }     
           case no_data_error: {
             /*  Tape started too far ahead of or behind the right sector.
+             *  This happens in the middle of a segment too !
              */
             update_history( cause);
-            if (buff->data_offset == 0) {
+            determine_progress( buff, cause);
+            if (buff->sector_count > 0) {
+              TRACEx1( 5, "partial success: %d sectors done up to now",
+                      buff->sector_count);
+            } else if (buff->data_offset == 0) {
               TRACE( 5, "sector not found: start problem");
               if (buff->retry > 1) {
                 ++buff->start_offset;
@@ -587,12 +623,14 @@ fdc_isr( void)
           case data_am_error: {
             update_history( cause);
             TRACE( 4, "single am/crc error");
+            determine_progress( buff, cause);
             retry_sector( &buff, cause, 1, buff->sector_offset);
             break;
           }
           case overrun_error: {
             update_history( cause);
             TRACE( 4, "single overrun error");
+            determine_progress( buff, cause);
             retry_sector( &buff, cause, 1, buff->sector_offset);
             break;
           }
@@ -666,7 +704,7 @@ fdc_isr( void)
   if (t0 >= 1000) {             /* only tell us about long calls */
     TRACEx1( 7, "isr() duration: %5d usec", t0);
   }
-  do_floppy = fdc_isr;          /* hook our handler into the fdc code again */
+  *fdc.hook = fdc_isr;          /* hook our handler into the fdc code again */
   TRACE_EXIT;
   --isr_active;
 }
