@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/ftape-eof.c,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/ftape-eof.c,v $
  $Author: bas $
  *
- $Revision: 1.7 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.10 $
+ $Date: 1994/06/29 20:11:06 $
  $State: Beta $
  *
  *      This file contains the reading and writing code
@@ -27,7 +27,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-eof.c,v 1.7 1994/05/26 23:43:45 bas Beta bas $";
+"$Id: ftape-eof.c,v 1.10 1994/06/29 20:11:06 bas Beta $";
 
 
 #include <linux/string.h>
@@ -47,6 +47,7 @@ static char RCSid[] =
 /*      Global vars.
  */
 int failed_sector_log_changed = 0;
+int eof_mark = 0;
 
 /*      Local vars.
  */
@@ -168,15 +169,15 @@ put_file_mark_in_map( unsigned segment, unsigned sector)
     }
   }
   if (entry == new_entry) {
-    TRACEx1( 5, "0x%04x already exists", new_entry);
+    TRACEx1( 5, "0x%04lx already exists", new_entry);
   } else {
     if (i < failed_sector_count) {
-      TRACEx3( 5, "insert 0x%08x at index %d before 0x%08x",
+      TRACEx3( 5, "insert 0x%08lx at index %d before 0x%08lx",
               new_entry, i, entry);
       memmove( &failed_sector_log[ i + 1], &failed_sector_log[ i],
               (failed_sector_count - i) * sizeof( *failed_sector_log));
     } else {
-      TRACEx1( 5, "appending 0x%08x", new_entry);
+      TRACEx1( 5, "appending 0x%08lx", new_entry);
     }
     failed_sector_log[ i].entry = new_entry;
     ++failed_sector_count;
@@ -395,3 +396,46 @@ ftape_seek_eof( unsigned count)
   TRACE_EXIT;
   return result;
 }
+
+
+/*
+ * Not convinced, but this should find the correct file number. - Jon
+ */
+int
+ftape_file_no( daddr_t *f_no, daddr_t *b_no)
+{
+  TRACE_FUN( 5, "ftape_file_no");
+  int result = 0;  
+  int i;
+
+  if (first_data_segment == -1) {
+    result = read_header_segment( deblock_buffer);
+  }
+  if (result < 0) {
+    *f_no = 0;
+    *b_no = 0;
+    TRACE_EXIT;
+    return result;
+  }
+  for (i = 0 ; i <= failed_sector_count; ++i) {
+    if ((ftape_seg_pos - 1) < failed_sector_log[ i].mark.segment) {
+      /* check if we have been positioned on the EOF mark */
+      if (eof_mark && i > 0) {
+        i--;
+      }
+      *f_no = i;
+      if (i == 0) {
+        *b_no = ftape_seg_pos - 2; /* one sector for header? */
+      }	else {
+        *b_no = ftape_seg_pos - failed_sector_log[i - 1].mark.segment - 1;
+      }
+      TRACE_EXIT;
+      return result;
+    }
+  }
+  /* we get here on a newly erased tape */
+  *f_no = 0;
+  *b_no = 0;  
+  TRACE_EXIT;
+  return result;
+}
