@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /usr/src/distr/ftape-1.12/RCS/ftape-eof.h,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/ftape-eof.h,v $
  $Author: bas $
  *
- $Revision: 1.6 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.8 $
+ $Date: 1994/06/29 20:11:06 $
  $State: Beta $
  *
  *      Definitions and declarations for the end of file markers
@@ -32,7 +32,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 /*      ftape-eof.c defined global vars.
  */
-int failed_sector_log_changed;
+extern int failed_sector_log_changed;
+extern int eof_mark;
 
 /*      ftape-eof.c defined global functions.
  */
@@ -46,5 +47,5 @@ extern void extract_file_marks( byte* address);
 extern int update_failed_sector_log( byte* buffer);
 extern int ftape_seek_eom( void);
 extern int ftape_seek_eof( unsigned count);
-
+extern int ftape_file_no( daddr_t *file, daddr_t *block);
 #endif
