@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/ftape-io.c,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/ftape-io.c,v $
  $Author: bas $
  *
- $Revision: 1.30 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.35 $
+ $Date: 1994/06/29 20:09:12 $
  $State: Beta $
  *
  *      This file contains some general functions
@@ -27,7 +27,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-io.c,v 1.30 1994/05/26 23:43:45 bas Beta bas $";
+"$Id: ftape-io.c,v 1.35 1994/06/29 20:09:12 bas Beta $";
 
 
 #include <linux/errno.h>
@@ -43,13 +43,12 @@ static char RCSid[] =
 #include "ftape-io.h"
 #include "ftape-rw.h"
 #include "ftape-write.h"
+#include "ftape-read.h"
 #include "ftape-eof.h"
 #include "kernel-interface.h"
 #include "calibr.h"
 #include "vendors.h"
 
-extern void (*do_floppy)(void); /* declared in ../../blk_drv/blk.h */
-
 /*      Global vars.
  */
 /* NOTE: sectors start numbering at 1, all others at 0 ! */
@@ -83,7 +82,18 @@ static int ftape_udelay_time;
 static wakeup_method methods[] = WAKEUP_METHODS;
 static vendor_struct drive_type = { UNKNOWN_VENDOR,
                                     unknown_wake_up, "Unknown" };
-
+static int ftape_data_rate = 0;
+static ftape_offline = 0;
+static const struct {
+  char* text;
+  int fdc_code;
+  byte drive_code;
+} rates[ 4] = {
+  { "2 M",   -1, /*unsupported*/ QIC_CONFIG_RATE_2000 },
+  { "1 M",   fdc_data_rate_1000, QIC_CONFIG_RATE_1000 },
+  { "500 K", fdc_data_rate_500,  QIC_CONFIG_RATE_500 },
+  { "250 K", fdc_data_rate_250,  QIC_CONFIG_RATE_250 },
+};  
 
 void
 udelay( int usecs)
@@ -121,7 +131,7 @@ ftape_sleep( unsigned int time) {
     TRACEx2( 8, "%d msec, %d ticks", time, ticks);
     current->timeout = jiffies + ticks;
     current->state = TASK_INTERRUPTIBLE;
-    save_flags(flags);
+    save_flags( flags);
     sti();
     do {
       while (current->state != TASK_RUNNING) {
@@ -132,7 +142,7 @@ ftape_sleep( unsigned int time) {
         break;                  /* exit on signal */
       }
     } while (current->timeout > 0);
-    restore_flags(flags);
+    restore_flags( flags);
   }
   TRACE_EXIT;
 }
@@ -213,34 +223,46 @@ ftape_parameter( int command)
 
 /*      Wait for the drive to get ready.
  *      timeout time in milli-seconds
+ *      Returned status is valid if result != -EIO
  */
 int
 ftape_ready_wait( int timeout, int* status)
 {
   TRACE_FUN( 8, "ftape_ready_wait");
   int result;
+  unsigned long t0;
 
   for (;;) {
+    t0 = jiffies;
     result = ftape_report_raw_drive_status( status);
     if (result < 0) {
       TRACE( 1, "ftape_report_raw_drive_status failed");
-      TRACE_EXIT;
-      return -EIO;
+      result = -EIO;
+      break;
     }
     if (*status & QIC_STATUS_READY) {
-      TRACE_EXIT;
-      return 0;
+      result = 0;
+      break;
     }
+    /* this will fail when jiffies wraps around about
+     * once every year :-)
+     */
+    timeout -= ((jiffies - t0) * SECOND) / HZ;
     if (timeout <= 0) {
       TRACE( 1, "timeout");
-      TRACE_EXIT;
-      return -ETIME;
+      result = -ETIME;
+      break;
     }
     ftape_sleep( SECOND / 10);
     timeout -= SECOND / 10;
+    if (current->signal & ~current->blocked) {
+      TRACE( 1, "awoken by non-blocked signal :-(");
+      result = -EINTR;
+      break;                    /* exit on signal */
+    }
   }
   TRACE_EXIT;
-  return 0;
+  return result;
 }
 
 /*      Issue command and wait up to timeout seconds for drive ready
@@ -542,6 +564,77 @@ ftape_report_tape_status( int* status)
   return (result < 0) ? -EIO : 0;
 }
 
+int
+ftape_set_data_rate( int rate)
+{
+  TRACE_FUN( 8, "ftape_set_data_rate");
+  int status;
+  int result;
+  int i;
+
+  if (rates[ rate].fdc_code == -1) {
+    TRACEx1( 1, "%sbps data rate not supported by the fdc",
+            rates[ rate].text);
+    result = -EINVAL;
+  } else {
+    for (i = 0 ; i < 2 ; ++i) {
+      result = ftape_command( QIC_SELECT_RATE);
+      if (result >= 0) {
+        result = ftape_parameter_wait( rates[ rate].drive_code + 2,
+                                      1 * SECOND, &status);
+        if (result >= 0) {        
+          result = ftape_report_raw_drive_status( &status);
+          if (result >= 0) {
+            if (status & QIC_STATUS_ERROR) {
+              int error;
+              int command;
+              
+              result = ftape_report_error( &error, &command);
+              if (result >= 0  && error == 31 && command == QIC_SELECT_RATE) {
+                TRACEx1( 1, "%sbps data rate not allowed by the drive",
+                        rates[ rate].text);
+                result = -EINVAL;
+                break;
+              }
+            } else {
+              break;            /* new rate successfully selected */
+            }
+          }
+        }
+      }
+    }
+  }
+  if (result != -EINVAL) {
+    if (result < 0) {
+      TRACE( 1, "Error selecting data rate");
+      result = -EIO;
+    } else {
+      /*  Some older drives do not support the report_drive_status
+       *  command, so we cannot verify the set rate.
+       *  Assume they are all able to handle 500 Kbps, so skip the
+       *  verification if called with this value.
+       */
+      result = ftape_report_configuration( &status);
+      if (result < 0) {
+        result = (rates[ rate].drive_code == QIC_CONFIG_RATE_500) ? 0 : -EINVAL;
+      } else {
+        status = (status & QIC_CONFIG_RATE_MASK) >> QIC_CONFIG_RATE_SHIFT;
+        if (status != rates[ rate].drive_code) {
+          TRACE( 1, "Drive didn't accept requested data rate");
+          result = -EIO;
+        }
+      }
+      if (result >= 0) {
+        fdc_set_data_rate( rates[ rate].fdc_code);
+        TRACEx1( 4, "Data rates set to %sbps", rates[ rate].text);
+        ftape_data_rate = rate; /* store rate set */
+      }
+    }
+  }
+  TRACE_EXIT;
+  return result;
+}
+
 /*      Seek the head to the specified track.
  */
 int
@@ -560,7 +653,7 @@ ftape_seek_head_to_track( int track)
     if (result < 0) {
       TRACE( 1, "ftape_command failed");
     } else {
-      result = ftape_parameter_wait( track + 2, 5 * SECOND, &status);
+      result = ftape_parameter_wait( track + 2, 20 * SECOND, &status);
       if (result < 0) {
         TRACE( 1, "ftape_parameter_wait failed");
       } else {
@@ -709,26 +802,10 @@ ftape_abort_operation( int* location)
     /* set timeout so that the tape will run into at logical EOT
      * if we missed the last sector and there are no queue pulses.
      */
-#if 0
-    result = fdc_interrupt_wait( 4 * SECOND);
-    if (result == 0) {
-      result = ftape_smart_stop( &location_known, location);
-      if (result == 0) {
-        runner_status = idle;
-      }
-    } else if (result == -ETIME) {
-      break;
-    } else {
-      TRACE( 1, "fdc_interrupt_wait failed");
-      TRACE_EXIT;
-      return result;
-    }
-#else
     result = ftape_smart_stop( &location_known, location);
     if (result == 0) {
       runner_status = idle;
     }
-#endif
   }
   if (runner_status != idle) {
     if (runner_status == do_abort) {
@@ -747,34 +824,6 @@ ftape_abort_operation( int* location)
   return result;
 }
 
-int
-ftape_reset_drive( void)
-{
-  int result = 0;
-  int i;
-
-  /*    We want to re-establish contact with our drive.
-   *    Must get it to reset itself, not that easy !
-   *    First try if the recalibrate will do (it will
-   *    correct current_cylinder is it was wrong !)
-   *    then fire a number of reset commands (single
-   *    step pulses) and pray for success.
-   */
-  fdc_recalibrate();
-  ftape_sleep( 10 * MILLISECOND);
-  for (i = 0; i < 3; ++i) {
-    result = ftape_command( QIC_RESET);
-    ftape_sleep( 10 * MILLISECOND);
-    if (result == 0) {
-      break;
-    }
-  }
-  if (result == 0) {
-    ftape_sleep( 10 * SECOND);
-  }
-  return result;
-}
-
 int
 ftape_wakeup_drive( vendor_struct drive_type)
 {
@@ -868,9 +917,72 @@ ftape_detach_drive( void)
   ftape_put_drive_to_sleep( drive_type);
   fdc_catch_stray_interrupts( 1);   /* one always comes */
   fdc_disable();
+  fdc_release_irq_and_dma();
   TRACE_EXIT;
 }
 
+int
+ftape_reset_drive( void)
+{
+  TRACE_FUN( 4, "ftape_reset_drive");
+  int result = 0;
+  int status;
+  int err_code;
+  int err_command;
+  int i;
+
+  /*    We want to re-establish contact with our drive.
+   *    Fire a number of reset commands (single step pulses)
+   *    and pray for success.
+   */
+  for (i = 0; i < 2; ++i) {
+    TRACE( 5, "Resetting fdc");
+    fdc_reset();
+    ftape_sleep( 10 * MILLISECOND);
+    TRACE( 5, "Reset command to drive");
+    result = ftape_command( QIC_RESET);
+    if (result == 0) {
+      ftape_sleep( 1 * SECOND);   /* drive not accessible during 1 second */
+      TRACE( 5, "Re-selecting drive");
+      /*  Strange, the QIC-117 specs don't mention this but the
+       *  drive gets deselected after a soft reset !
+       *  So we need to enable it again.
+       */
+      result = ftape_wakeup_drive( drive_type);
+      if (result < 0) {
+        TRACE( 1, "Wakeup failed !");
+      }
+      TRACE( 5, "Waiting until drive gets ready");
+      result = ftape_ready_wait( 105 * SECOND, &status);
+      if (result == 0  && status & QIC_STATUS_ERROR) {
+        result = ftape_report_error( &err_code, &err_command);
+        if (result == 0 && err_code == 27) {
+          /* Okay, drive saw reset command and responded as it should
+           */
+          break;
+        } else {
+          result = -EIO;
+        }
+      } else {
+        result = -EIO;
+      }
+    }
+    if (current->signal & ~current->blocked) {
+      TRACE( 1, "aborted by signal");
+      result = -EINTR;
+      break;                  /* exit on signal */
+    }
+  }
+  if (result != 0) {
+    TRACE( 1, "General failure to reset tape drive");
+  }
+  /*  Restore correct settings
+   */
+  ftape_set_data_rate( ftape_data_rate); /* keep original rate */
+  TRACE_EXIT;
+  return result;
+}
+
 /*      OPEN routine called by kernel-interface code
  */
 int
@@ -885,17 +997,12 @@ _ftape_open( void)
   int status;
   static int new_tape = 1;
   wake_up_types wake_up = unknown_wake_up;
+  int rate;
 
-  do_floppy = fdc_isr;
-  ftape_motor = 0;
-  fdc_catch_stray_interrupts( 1); /* one always comes */
-  TRACE( 5, "resetting fdc");
-  fdc_reset();                  /* init fdc */
-  TRACE( 5, "recalibrating");
-  if (fdc_recalibrate() != 0) {
-    TRACE( 1, "recalibrate failed");
+  result = fdc_init();
+  if (result < 0) {
     TRACE_EXIT;
-    return -EIO;
+    return result;
   }
   /* If we already know the drive type, wake it up.
    * Else try to find out what kind of drive is attached.
@@ -905,6 +1012,7 @@ _ftape_open( void)
     result = ftape_wakeup_drive( drive_type);
     if (result < 0) {
       TRACE( 1, "known wakeup method failed");
+      fdc_release_irq_and_dma();
       TRACE_EXIT;
       return result;
     }
@@ -933,6 +1041,7 @@ _ftape_open( void)
       drive_type.wake_up = unknown_wake_up;
       TRACE( 1, "no tape drive found !");
       tracing = old_tracing;
+      fdc_release_irq_and_dma();
       TRACE_EXIT;
       return -ENODEV;
     }
@@ -941,11 +1050,30 @@ _ftape_open( void)
    *    First clear error status if present.
    */
   do {
-    ftape_ready_wait( 105 * SECOND, &status);
+    result = ftape_ready_wait( 105 * SECOND, &status);
+    if (result < 0) {
+      if (result == -ETIME) {
+        TRACE( 1, "ftape_ready_wait timeout");
+#ifdef SOFT_RESET
+        result = ftape_reset_drive();
+        if (result == 0) {
+          continue;
+        }
+#endif
+      } else if (result == -EINTR) {
+        TRACE( 1, "ftape_ready_wait aborted");
+      } else {
+        TRACE( 1, "ftape_ready_wait failed");
+      }
+      ftape_detach_drive();
+      TRACE_EXIT;
+      return -EIO;
+    }
     /*  Exit if no tape inserted
      */
     if ((status & QIC_STATUS_CARTRIDGE_PRESENT) == 0) {
       TRACE( 1, "no cartridge present");
+      ftape_detach_drive();
       TRACE_EXIT;
       return -EIO;
     }
@@ -957,10 +1085,11 @@ _ftape_open( void)
       }
       result = ftape_report_error( &error.error, &error.command);
       if (result < 0) {
-        TRACE( 1, "report_error_code failed");
+        TRACEi( 1, "report_error_code failed:", result);
         ftape_reset_drive();    /* hope it's working next time */
+        fdc_release_irq_and_dma();
         TRACE_EXIT;
-        return result;
+        return -EIO;
       }
       TRACEi( 4, "error code   :", error.error);
       TRACEi( 4, "error command:", error.command);
@@ -973,13 +1102,18 @@ _ftape_open( void)
 
   write_protected = !!(status & QIC_STATUS_WRITE_PROTECT);
   if (write_protected) {
-    TRACE( 2, "warning: cartridge write protected");
+    TRACE( 2, "Write protected cartridge");
   }
   if (drive_type.vendor_id == UNKNOWN_VENDOR) {
     int vendor_index;
 
     ftape_report_vendor_id( &drive_type.vendor_id);
     vendor_index = lookup_vendor_id( drive_type.vendor_id);
+    if (drive_type.vendor_id == UNKNOWN_VENDOR && 
+        drive_type.wake_up == wake_up_colorado) {
+      vendor_index = 0;
+      drive_type.vendor_id = 0; /* hack to get rid of all this mail */
+    }
     if (vendor_index < 0) {
       /* Unknown vendor id, first time opening device.
        * The drive_type remains set to type found at wakeup time, this
@@ -1021,16 +1155,41 @@ _ftape_open( void)
         return result;
       } else if (!(status & QIC_STATUS_REFERENCED)) {
         TRACE( 1, "warning: tape not formatted");
-#if 0
+        /*  May need to change this when we want to be able to
+         *  reference a bad tape for formatting purposes.
+         */
+        ftape_detach_drive();
         TRACE_EXIT;
         return -EIO;
-#endif
       }
     }
+#ifdef DATA_RATE
+    rate = DATA_RATE;
+#else
+    rate = (fdc.type == i8272) ? 2 : 1;
+#endif
+    for ( ; rate < 4 ; ++rate) {
+      result = ftape_set_data_rate( rate);
+      if (result >= 0) {
+        break;
+      }
+    }
+    if (result < 0) {
+      ftape_detach_drive();
+      TRACE_EXIT;
+      return -EIO;
+    }
     /* Tape should be at bot if new cartridge ! */
     ftape_new_cartridge();
     ftape_get_tape_parameters();
     new_tape = 0;
+  } else {
+    if (ftape_offline) {
+      TRACE( 2, "Tape drive offline !");
+      ftape_detach_drive();
+      TRACE_EXIT;
+      return -EBUSY;            /* well, offline would be better */
+    }
   }
   history.used = 0;
   history.id_am_errors = 0;
@@ -1056,17 +1215,19 @@ _ftape_close( void)
 {
   TRACE_FUN( 8, "_ftape_close");
   int result = 0;
-  int last_segment;
+  int last_segment = ftape_seg_pos - 1;
 
-  result = ftape_flush_buffers();
-  last_segment = ftape_seg_pos - 1;
-  if (result >= 0) {
-    result = ftape_update_header_segments( NULL, 1);
-    if (result < 0) {
-      TRACE( 1, "error: update of header segments failed");
+  if (!write_protected) {
+    result = ftape_flush_buffers();
+    last_segment = ftape_seg_pos - 1; /* changed ? */
+    if (result >= 0) {
+      result = ftape_update_header_segments( NULL, 1);
+      if (result < 0) {
+        TRACE( 1, "error: update of header segments failed");
+      }
+    } else {
+      TRACE( 1, "error: unable to update header segments");
     }
-  } else {
-    TRACE( 1, "error: unable to update header segments");
   }
   ftape_abort_operation( &current_segment);
   if ((ftape_unit & FTAPE_NO_REWIND) == 0) {
@@ -1074,6 +1235,14 @@ _ftape_close( void)
     result = ftape_seek_to_bot();
     ftape_reset_position();
     ftape_zap_buffers();
+  } 
+  /* Gnu tar stops on the EOF as it knows there is no more data
+   * and doesn't read the EOF, so inorder the next open doesn't start
+   * on an EOF we clear the state of eof_mark now.
+   */
+  if (just_before_eof) {        /* nothing was left */
+    eof_mark = 0;
+    just_before_eof = 0;
   }
   ftape_detach_drive();
   if (history.used) {
@@ -1106,8 +1275,11 @@ int
 _ftape_ioctl( unsigned int command, void * arg)
 {
   TRACE_FUN( 8, "ftape_ioctl");
+  int gen_status;  
   int result = EINVAL;
-  struct mtop krnl_arg;
+  union { struct mtop mtop;
+          struct mtget mtget;
+  } krnl_arg;
   int arg_size = (command & IOCSIZE_MASK) >> IOCSIZE_SHIFT;
 
   /* This check will only catch arguments that are too large !
@@ -1117,33 +1289,35 @@ _ftape_ioctl( unsigned int command, void * arg)
     TRACE_EXIT;
     return -EINVAL;
   }
-
   if (command & IOC_IN) {
     int error = verify_area( VERIFY_READ, arg, arg_size);
     if (error) {
       TRACE_EXIT;
       return error;
     }
-    memcpy_fromfs( &krnl_arg, arg, arg_size);
+    memcpy_fromfs( &krnl_arg.mtop, arg, arg_size);
   }
-
-  TRACElx( 5, "called with command:", command);
-
+  TRACEx1( 5, "called with ioctl command: 0x%08x", command);
   switch (command) {
     /* cpio compatibility
      * mtrasx and mtreset are mt extension by Hennus Bergman
      * mtseek and mttell are mt extension by eddy olk
      */
   case MTIOCTOP:
-    TRACE( 5, "Mag tape ioctl command: MTIOCTOP");
-    switch (krnl_arg.mt_op) {
+    TRACEx1( 5, "calling MTIOCTOP command: 0x%08x", krnl_arg.mtop.mt_op);
+    switch (krnl_arg.mtop.mt_op) {
     case MTRESET:
       result = ftape_reset_drive();
       /* fall through */
     case MTREW:
+    case MTOFFL:
       result = ftape_seek_to_bot();
       ftape_reset_position();
       ftape_zap_buffers();
+      if (krnl_arg.mtop.mt_op == MTOFFL) {
+        ftape_offline = 1;
+        TRACE( 4, "Putting tape drive offline");
+      }
       result = 0;
       break;
     case MTRETEN:
@@ -1158,23 +1332,31 @@ _ftape_ioctl( unsigned int command, void * arg)
     case MTEOM:
       result = ftape_seek_eom();
       break;
+    case MTFSFM:
+      eof_mark = 1; /* position ready to extend */
     case MTFSF:
-      result = ftape_seek_eof( krnl_arg.mt_count);
+      result = ftape_seek_eof( krnl_arg.mtop.mt_count);
       break;
+    case MTBSFM:
+      eof_mark = 1; /* position ready to extend */    
     case MTBSF:
-      result = ftape_seek_eof( -krnl_arg.mt_count);
+      result = ftape_seek_eof( -krnl_arg.mtop.mt_count);
       break;
     case MTFSR:
-      tracing = krnl_arg.mt_count;
+      tracing = krnl_arg.mtop.mt_count;
       result = 0;
       break;
     case MTBSR:
+#if 0
       result = ftape_fix();
+#else
+      result = 0;
+#endif
       break;
     case MTWEOF:
-      result = ftape_weof( krnl_arg.mt_count, ftape_seg_pos, 1);
+      result = ftape_weof( krnl_arg.mtop.mt_count, ftape_seg_pos, 1);
       break;
-    case MTOFFL:
+    case MTNOP:      /* required for gnu mt */
       result = 0;
       break;
       /* MTRASx and MTRESET are mt extension by Hennus Bergman
@@ -1184,18 +1366,28 @@ _ftape_ioctl( unsigned int command, void * arg)
     case MTRAS3:
     case MTSEEK:
     case MTTELL:
-    case MTNOP:
-    case MTFSFM:
-    case MTBSFM:
     default:
-      TRACEi( 1, "MTIOCTOP sub-command not implemented:", krnl_arg.mt_op);
+      TRACEi( 1, "MTIOCTOP sub-command not implemented:", krnl_arg.mtop.mt_op);
       result = -EIO;
       break;
     }
     break;
   case MTIOCGET:
-    TRACE( 5, "Mag tape ioctl command: MTIOCGET");
-    TRACE( 1, "MTIOCGET command not implemented");
+    krnl_arg.mtget.mt_type = drive_type.vendor_id + 0x800000;
+
+    gen_status = 0x01000000; /* how can we ever be off line? */
+    gen_status |= write_protected ? 0x04000000 : 0;
+    
+    krnl_arg.mtget.mt_resid = 0;     /* hmmm ? */
+    krnl_arg.mtget.mt_dsreg = 0;      /* status register */
+    krnl_arg.mtget.mt_gstat = gen_status;  /* device independent status */
+    krnl_arg.mtget.mt_erreg = 0;      /* error register */
+  
+    /* This is wrong, and block number is in segments...
+     * Whats a block on a tape streamer?
+     */
+    result = ftape_file_no(&krnl_arg.mtget.mt_fileno,
+			   &krnl_arg.mtget.mt_blkno); 
     break;
   case MTIOCPOS:
     TRACE( 5, "Mag tape ioctl command: MTIOCPOS");
