@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/ftape-read.c,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/ftape-read.c,v $
  $Author: bas $
  *
- $Revision: 1.7 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.9 $
+ $Date: 1994/06/29 20:11:06 $
  $State: Beta $
  *
  *      This file contains the reading code
@@ -27,7 +27,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-read.c,v 1.7 1994/05/26 23:43:45 bas Beta bas $";
+"$Id: ftape-read.c,v 1.9 1994/06/29 20:11:06 bas Beta $";
 
 
 #include <linux/string.h>
@@ -50,7 +50,7 @@ static char RCSid[] =
  */
 int buf_pos_rd = 0;
 int buf_len_rd = 0;
-int eof_mark = 0;
+int just_before_eof = 0;
 
 /*      Local vars.
  */
@@ -614,6 +614,7 @@ _ftape_read( char* buff, int req_len)
     buf_pos_rd += cnt;          /* index in buffer */
     buf_len_rd -= cnt;          /* remaining bytes in buffer */
     if (eof_mark && buf_len_rd == 0) { /* nothing left */
+      just_before_eof = 1;
       TRACE( 5, "partial count because of eof mark");
       if (bytes_read == 0) {
         eof_mark = 0;           /* no need for mark next read */
