@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /usr/src/distr/ftape-1.12/RCS/ftape-read.h,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/ftape-read.h,v $
  $Author: bas $
  *
- $Revision: 1.5 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.7 $
+ $Date: 1994/06/29 20:11:06 $
  $State: Beta $
  *
  *      This file contains the definitions for the read functions
@@ -35,7 +35,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 extern int buf_pos_rd;
 extern int buf_len_rd;
-extern int eof_mark;
+extern int just_before_eof;
 
 /*      ftape-read.c defined global functions.
  */
