@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/ftape-rw.c,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/ftape-rw.c,v $
  $Author: bas $
  *
- $Revision: 1.30 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.32 $
+ $Date: 1994/06/29 20:11:06 $
  $State: Beta $
  *
  *      This file contains some common code for the segment read and segment
@@ -27,7 +27,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-rw.c,v 1.30 1994/05/26 23:43:45 bas Beta bas $";
+"$Id: ftape-rw.c,v 1.32 1994/06/29 20:11:06 bas Beta $";
 
 
 #include <linux/string.h>
@@ -495,8 +495,17 @@ ftape_start_tape( int segment_id)
     if (ftape_track != track) {
       /* current unknown or not equal to destination
        */
+      int save_track = ftape_track;
+
       TRACEi( 5, "seeking head to track", track);
-      ftape_seek_head_to_track( track);
+      result = ftape_ready_wait( 15 * SECOND, &status);
+      if (result >= 0) {
+        result = ftape_seek_head_to_track( track);
+      }
+      if (result < 0) {
+        ftape_track = save_track;
+        TRACE( 2, "seeking head to track failed, may cause big trouble !");
+      }
       location_known = 0;
     }
     at_end_of_track = 0;
@@ -560,8 +569,17 @@ ftape_start_tape( int segment_id)
         tape_running = 0;
       }
       TRACE( 5, "repositioning");
-      fast_seek( -(rewind_gaps + reverse_margin +
-                buffer[ head].start_offset - 1));
+      if (segment_id % segments_per_track == 0) {
+        /*  If seeking to first segment on track better do a complete
+         *  rewind to logical begin of track to get a more steady tape
+         *  motion.
+         */
+        result = ftape_command_wait( (ftape_track & 1) ? QIC_PHYSICAL_FORWARD :
+                                    QIC_PHYSICAL_REVERSE, 85 * SECOND, &status);
+      } else {
+        fast_seek( -(rewind_gaps + reverse_margin +
+                     buffer[ head].start_offset - 1));
+      }
       ++history.rewinds;
       next_segment = segment_id - reverse_margin - buffer[ head].start_offset;
       result = ftape_ready_wait( 15 * SECOND, &status);
