@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/ftape-write.c,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/ftape-write.c,v $
  $Author: bas $
  *
- $Revision: 1.7 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.9 $
+ $Date: 1994/06/29 20:11:06 $
  $State: Beta $
  *
  *      This file contains the writing code
@@ -27,7 +27,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-write.c,v 1.7 1994/05/26 23:43:45 bas Beta bas $";
+"$Id: ftape-write.c,v 1.9 1994/06/29 20:11:06 bas Beta $";
 
 
 #include <linux/string.h>
@@ -732,15 +732,26 @@ ftape_fix( void)
   int dummy;
   int status;
 
-  /*  This will copy header segment 2 to header segment 1
-   *  Spares us a tape format operation if header 2 is still good.
-   */
-  header_segment_1 = 0;
-  header_segment_2 = 1;
-  first_data_segment = 2;
-  result = read_segment( header_segment_2, scratch_buffer, &dummy, 0);
-  result = ftape_ready_wait( 5 * SECOND, &status);
-  result = ftape_write_header_segments( scratch_buffer);
+  if (write_protected) {
+    result = -EROFS;
+  } else {
+    /*  This will copy header segment 2 to header segment 1
+     *  Spares us a tape format operation if header 2 is still good.
+     */
+    header_segment_1 = 0;
+    header_segment_2 = 1;
+    first_data_segment = 2;
+    result = read_segment( header_segment_2, scratch_buffer, &dummy, 0);
+    result = ftape_ready_wait( 5 * SECOND, &status);
+    result = ftape_write_header_segments( scratch_buffer);
+  }
   TRACE_EXIT;
   return result;
 }
+
+
+
+
+
+
+
