@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-1.12/RCS/kernel-interface.c,v $
+ $Source: /usr/src/distr/ftape-1.13b/RCS/kernel-interface.c,v $
  $Author: bas $
  *
- $Revision: 1.26 $
- $Date: 1994/05/26 23:42:56 $
+ $Revision: 1.32 $
+ $Date: 1994/07/05 20:18:02 $
  $State: Beta $
  *
  *      This file contains the code that interfaces the kernel
@@ -27,7 +27,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: kernel-interface.c,v 1.26 1994/05/26 23:42:56 bas Beta bas $";
+"$Id: kernel-interface.c,v 1.32 1994/07/05 20:18:02 bas Beta bas $";
 
 
 #include <linux/errno.h>
@@ -91,7 +91,7 @@ int
 init_module( void) {
   TRACE_FUN( 5, "init_module");
 
-  printk( "ftape v0.9.11 25/5/94 (c) 1993, 1994 Bas Laarhoven (bas@vimec.nl)"
+  printk( "ftape v1.13b 5/7/94 (c) 1993, 1994 Bas Laarhoven (bas@vimec.nl)"
          "\n QIC-117 driver for QIC-40 and QIC-80 tape drives\n"
 #if 0
          "====== this is not an official release, don't distribute or\n"
@@ -125,11 +125,13 @@ void
 cleanup_module( void) {
   TRACE_FUN( 5, "cleanup_module");
 
-  do_floppy = NULL;             /* prevent accidents ! */
+  if (fdc.hook == &do_floppy) {
+    do_floppy = NULL;           /* prevent accidents ! */
+  }
   if (unregister_chrdev( QIC117_TAPE_MAJOR, "mt") != 0) {
     TRACE( 3, "failed");
   } else {
-    TRACE( 3, "successfull");
+    TRACE( 3, "successful");
   }
   TRACE_EXIT;
 }
