@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /usr/src/distr/ftape-1.12/RCS/qic117.h,v $
+ $Source: /usr/src/distr/ftape-1.13a/RCS/qic117.h,v $
  $Author: bas $
  *
- $Revision: 1.19 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.21 $
+ $Date: 1994/06/29 20:11:06 $
  $State: Beta $
  *
  *      This file contains QIC-117 spec. related definitions
@@ -100,7 +100,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 #define QIC117_COMMANDS {\
 /* 0*/  NULL,\
-/* 1*/  "reset",\
+/* 1*/  "soft reset",\
 /* 2*/  "report next bit",\
 /* 3*/  "pause",\
 /* 4*/  "micro step pause",\
@@ -233,9 +233,11 @@ struct qic117_command_restriction {
  */
 
 #define QIC_CONFIG_RATE_MASK	0x18
-#define QIC_CONFIG_RATE_250	0x00
-#define QIC_CONFIG_RATE_500	0x10
-#define QIC_CONFIG_RATE_1000	0x18
+#define QIC_CONFIG_RATE_SHIFT	3
+#define QIC_CONFIG_RATE_250	0
+#define QIC_CONFIG_RATE_500	2
+#define QIC_CONFIG_RATE_1000	3
+#define QIC_CONFIG_RATE_2000	1
 
 #define QIC_CONFIG_LONG		0x40 /* Extra Length Tape Detected */
 #define QIC_CONFIG_80		0x80 /* QIC-80 detected. */
@@ -255,7 +257,7 @@ struct qic117_command_restriction {
 #define QIC_TAPE_295FT          0x30
 
 /*
- *      Errors: List of error codes, and their fatalness.
+ *      Errors: List of error codes, and their severity.
  */
 
 typedef struct {
