@@ -19,11 +19,11 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /usr/src/distr/ftape-1.12/RCS/vendors.h,v $
+ $Source: /usr/src/distr/ftape-1.13/RCS/vendors.h,v $
  $Author: bas $
  *
- $Revision: 1.16 $
- $Date: 1994/05/26 23:43:45 $
+ $Revision: 1.18 $
+ $Date: 1994/06/23 04:51:23 $
  $State: Beta $
  *
  *      This file contains the supported drive types
@@ -65,19 +65,22 @@ typedef struct {
 
 #define QIC117_VENDORS { \
 /* see _vendor_struct */ \
+  { 0x00000, wake_up_colorado,  "Colorado DJ-10 (old)" }, \
   { 0x00047, wake_up_colorado,  "Colorado DJ-10/DJ-20" }, \
   { 0x011c4, wake_up_colorado,  "Colorado DJ-10/DJ-20 (new)" }, \
   { 0x00005, wake_up_mountain,  "Archive 5580i" }, \
   { 0x10005, wake_up_insight,   "Insight 80 Mb" }, \
+  { 0x00140, wake_up_mountain,  "Identity IDTBU120E, Escom ?" }, \
+  { 0x00146, wake_up_mountain,  "Escom/Archive 31250Q" }, \
   { 0x0014a, wake_up_mountain,  "Archive XL9250i [Conner/Escom]" }, \
   { 0x0014c, wake_up_mountain,  "Conner C250MQT" }, \
+  { 0x0014e, wake_up_mountain,  "Conner C250MQ" }, \
+  { 0x00180, wake_up_mountain,  "Summit SE 150" }, \
+  { 0x00181, wake_up_mountain,  "Summit SE 250, Mountain FS8000" }, \
   { 0x001c1, no_wake_up,        "Wangtek 3040F" }, \
   { 0x001c8, wake_up_colorado,  "Wangtek 3080F" }, \
   { 0x08880, no_wake_up,        "Iomega 250" }, \
-  { 0x00146, wake_up_mountain,  "Escom/Archive 31250Q" }, \
-  { 0x00180, wake_up_mountain,  "Summit SE 150" }, \
-  { 0x00181, wake_up_mountain,  "Summit SE 250, Mountain FS8000" }, \
-  { 0x00140, wake_up_mountain,  "Identity IDTBU120E, Escom ?" }, \
+  { 0x00021, no_wake_up,        "AIWA CT-803" }, \
 }
 
 #define QIC117_MAKE_CODES { \
